<?php

declare(strict_types=1);

namespace Drupal\rift_ui\Controller;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\media\MediaInterface;
use Drupal\rift\RiftPicture;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for Responsive Image Formatter tools UI routes.
 */
final class RiftUiEndpointController implements ContainerInjectionInterface {

  /**
   * The controller constructor.
   */
  public function __construct(
    private readonly RiftPicture $riftPicture,
    private readonly ConfigFactoryInterface $configFactory,
    private readonly RendererInterface $renderer,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('rift.picture'),
      $container->get('config.factory'),
      $container->get('renderer'),
    );
  }

  /**
   * Builds the response.
   */
  public function getMedia(MediaInterface $media) {
    $settings = $this->configFactory->get('rift.settings');
    $config = $settings->get('config');
    $view_modes = $settings->get('view_modes');
    $output = [];
    foreach ($view_modes as $key => $view_mode) {
      $runtime_config = $config;
      foreach ($view_mode as $k => $v) {
        $runtime_config[$k] = $v;
      }
      $build = $this->riftPicture->responsivePicture($media, $runtime_config);
      $output_string = $this->renderer->render($build);
      $output['view_modes'][$key] = [
        'html' => $output_string,
      ];
    }
    return new JsonResponse($output);
  }

  /**
   * Settings Read.
   */
  public function getSettings() {
    $settings = $this->configFactory->get('rift.settings');
    return new JsonResponse([
      'config' => $settings->get('config'),
      'view_modes' => $settings->get('view_modes'),
    ]);
  }

  /**
   * Settings write.
   */
  public function setSettings(Request $request) {
    $settings = $this->configFactory->getEditable('rift.settings');
    $content = $request->getContent();
    $decoded = Json::decode($content);
    $temp_settings = $settings->getRawData();
    $temp_settings['config'] = $decoded['config'];
    $temp_settings['view_modes'] = $decoded['view_modes'];
    $settings->setData($temp_settings);
    $settings->save(FALSE);
    return $this->getSettings();
  }

}
